
# Builtin imports
import unittest
import logging

# Internal imports
import root_pb2
from testing.test_utils import PatternTest
import logconf


class IntermediateSubexpressionTests(unittest.TestCase,
                              PatternTest):
    patterns = {
        r"\b[A-Z]{1}\b": PatternTest.gen_root(root_pb2.Expression(
            raw=r'\b[A-Z]{1}\b',
            tokens=[
                # Add tokens here
                root_pb2.Token(
                    token=r"\b",
                    type=root_pb2.TokenType.Anchor,
                    anchor=root_pb2.AnchorType.WordBoundry
                ),
                root_pb2.Token(
                    token="[",
                    type=root_pb2.TokenType.CharacterClass,
                    characterclass=root_pb2.CharacterClassType.OpenSet
                ),
                root_pb2.Token(
                    token="A-Z",
                    type=root_pb2.TokenType.CharacterClass,
                    characterclass=root_pb2.CharacterClassType.RangeSet
                ),
                root_pb2.Token(
                    token="]",
                    type=root_pb2.TokenType.CharacterClass,
                    characterclass=root_pb2.CharacterClassType.CloseSet
                ),
                root_pb2.Token(
                    token="{1}",
                    type=root_pb2.TokenType.QuantifierModifier,
                    quantifiermodifier=root_pb2.QuantifierModifierType.SpecifiedQuantifier
                ),
                root_pb2.Token(
                    token=r"\b",
                    type=root_pb2.TokenType.Anchor,
                    anchor=root_pb2.AnchorType.WordBoundry
                ),
            ],
            expressions=[
                root_pb2.Expression(
                    raw=r"\b",
                    tokens=[
                        root_pb2.Token(
                            token=r"\b",
                            type=root_pb2.TokenType.Anchor,
                            anchor=root_pb2.AnchorType.WordBoundry
                        ),
                    ]
                ),
                root_pb2.Expression(
                    raw="[A-Z]{1}",
                    tokens=[
                        root_pb2.Token(
                            token="[",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.OpenSet
                        ),
                        root_pb2.Token(
                            token="A-Z",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.RangeSet
                        ),
                        root_pb2.Token(
                            token="]",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.CloseSet
                        ),
                        root_pb2.Token(
                            token="{1}",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.SpecifiedQuantifier
                        ),
                    ]
                ),
                root_pb2.Expression(
                    raw=r"\b",
                    tokens=[
                        root_pb2.Token(
                            token=r"\b",
                            type=root_pb2.TokenType.Anchor,
                            anchor=root_pb2.AnchorType.WordBoundry
                        ),
                    ]
                ),
            ]
        )),

        r"/^(?:0x[a-f\d]+|0b[01]+|(?:\d+\.?\d*|\.\d+)(?:e[-+]?\d+)?)(u|ll?|l|f)?/i": PatternTest.gen_root(root_pb2.Expression(
            raw=r"/^(?:0x[a-f\d]+|0b[01]+|(?:\d+\.?\d*|\.\d+)(?:e[-+]?\d+)?)(u|ll?|l|f)?/i",
            tokens=[
                # Add tokens here
                root_pb2.Token(
                    token=r"/",
                    type=root_pb2.TokenType.Anchor,
                    anchor=root_pb2.AnchorType.ForwardSlash
                ),
                root_pb2.Token(
                    token="^",
                    type=root_pb2.TokenType.CharacterClass,
                    characterclass=root_pb2.CharacterClassType.SetNegation
                ),
                root_pb2.Token(
                    token=r"(",
                    type=root_pb2.TokenType.GroupReference,
                    groupref=root_pb2.GroupReferenceType.OpenCapture
                ),
                root_pb2.Token(
                    token=r"?:",
                    type=root_pb2.TokenType.GroupReference,
                    groupref=root_pb2.GroupReferenceType.NonCapturing
                ),
                root_pb2.Token(
                    token="0",
                    type=root_pb2.TokenType.Character,
                    character="0"
                ),
                root_pb2.Token(
                    token="x",
                    type=root_pb2.TokenType.Character,
                    character="x"
                ),
                root_pb2.Token(
                    token="[",
                    type=root_pb2.TokenType.CharacterClass,
                    characterclass=root_pb2.CharacterClassType.OpenSet
                ),
                root_pb2.Token(
                    token="a-f",
                    type=root_pb2.TokenType.CharacterClass,
                    characterclass=root_pb2.CharacterClassType.RangeSet
                ),
                root_pb2.Token(
                    token=r"\d",
                    type=root_pb2.TokenType.CharacterClass,
                    characterclass=root_pb2.CharacterClassType.Digit
                ),
                root_pb2.Token(
                    token="]",
                    type=root_pb2.TokenType.CharacterClass,
                    characterclass=root_pb2.CharacterClassType.CloseSet
                ),
                root_pb2.Token(
                    token="+",
                    type=root_pb2.TokenType.QuantifierModifier,
                    quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                ),
                root_pb2.Token(
                    token="|",
                    type=root_pb2.TokenType.QuantifierModifier,
                    quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                ),
                root_pb2.Token(
                    token="0",
                    type=root_pb2.TokenType.Character,
                    character="0"
                ),
                root_pb2.Token(
                    token="b",
                    type=root_pb2.TokenType.Character,
                    character="b"
                ),
                root_pb2.Token(
                    token="[",
                    type=root_pb2.TokenType.CharacterClass,
                    characterclass=root_pb2.CharacterClassType.OpenSet
                ),
                root_pb2.Token(
                    token="0",
                    type=root_pb2.TokenType.Character,
                    character="0"
                ),
                root_pb2.Token(
                    token="1",
                    type=root_pb2.TokenType.Character,
                    character="1"
                ),
                root_pb2.Token(
                    token="]",
                    type=root_pb2.TokenType.CharacterClass,
                    characterclass=root_pb2.CharacterClassType.CloseSet
                ),
                root_pb2.Token(
                    token="+",
                    type=root_pb2.TokenType.QuantifierModifier,
                    quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                ),
                root_pb2.Token(
                    token="|",
                    type=root_pb2.TokenType.QuantifierModifier,
                    quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                ),
                root_pb2.Token(
                    token=r"(",
                    type=root_pb2.TokenType.GroupReference,
                    groupref=root_pb2.GroupReferenceType.OpenCapture
                ),
                root_pb2.Token(
                    token="?:",
                    type=root_pb2.TokenType.GroupReference,
                    groupref=root_pb2.GroupReferenceType.NonCapturing
                ),
                root_pb2.Token(
                    token=r"\d",
                    type=root_pb2.TokenType.CharacterClass,
                    characterclass=root_pb2.CharacterClassType.Digit
                ),
                root_pb2.Token(
                    token="+",
                    type=root_pb2.TokenType.QuantifierModifier,
                    quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                ),
                root_pb2.Token(
                    token="\.",
                    type=root_pb2.TokenType.Escape,
                    escape=root_pb2.EscapeType.Reserved
                ),
                root_pb2.Token(
                    token="?",
                    type=root_pb2.TokenType.QuantifierModifider,
                    quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                ),
                root_pb2.Token(
                    token=r"\d",
                    type=root_pb2.TokenType.CharacterClass,
                    characterclass=root_pb2.CharacterClassType.Digit
                ),
                root_pb2.Token(
                    token="*",
                    type=root_pb2.TokenType.QuantifierModifider,
                    quantifiermodifier=root_pb2.QuantifierModifierType.Star
                ),
                root_pb2.Token(
                    token=r"\.",
                    type=root_pb2.TokenType.Anchor,
                    anchor=root_pb2.AnchorType.ForwardSlash
                ),
                root_pb2.Token(
                    token=r"\d",
                    type=root_pb2.TokenType.CharacterClass,
                    characterclass=root_pb2.CharacterClassType.Digit
                ),
                root_pb2.Token(
                    token="+",
                    type=root_pb2.TokenType.QuantifierModifier,
                    quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                ),
                root_pb2.Token(
                    token=")",
                    type=root_pb2.TokenType.GroupReference,
                    groupref=root_pb2.GroupReferenceType.CloseCapture
                ),
                root_pb2.Token(
                    token=r"(",
                    type=root_pb2.TokenType.GroupReference,
                    groupref=root_pb2.GroupReferenceType.OpenCapture
                ),
                root_pb2.Token(
                    token="?:",
                    type=root_pb2.TokenType.GroupReference,
                    groupref=root_pb2.GroupReferenceType.NonCapturing
                ),
                root_pb2.Token(
                    token="e",
                    type=root_pb2.TokenType.Character,
                    character="e"
                ),
                root_pb2.Token(
                    token="[",
                    type=root_pb2.TokenType.CharacterClass,
                    characterclass=root_pb2.CharacterClassType.OpenSet
                ),
                root_pb2.Token(
                    token="-",
                    type=root_pb2.TokenType.Character,
                    character="-"
                ),
                root_pb2.Token(
                    token="+",
                    type=root_pb2.TokenType.Character,
                    character="+"
                ),
                root_pb2.Token(
                    token="]",
                    type=root_pb2.TokenType.CharacterClass,
                    characterclass=root_pb2.CharacterClassType.CloseSet
                ),
                root_pb2.Token(
                    token="?",
                    type=root_pb2.TokenType.QuantifierModifier,
                    quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                ),
                root_pb2.Token(
                    token=r"\d",
                    type=root_pb2.TokenType.CharacterClass,
                    characterclass=root_pb2.CharacterClassType.Digit
                ),
                root_pb2.Token(
                    token="+",
                    type=root_pb2.TokenType.QuantifierModifier,
                    quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                ),
                root_pb2.Token(
                    token=")",
                    type=root_pb2.TokenType.GroupReference,
                    groupref=root_pb2.GroupReferenceType.ClosedCapture
                ),
                root_pb2.Token(
                    token="?",
                    type=root_pb2.TokenType.QuantifierModifier,
                    quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                ),
                root_pb2.Token(
                    token=")",
                    type=root_pb2.TokenType.GroupReference,
                    groupref=root_pb2.GroupReferenceType.ClosedCapture
                ),
                root_pb2.Token(
                    token=r"(",
                    type=root_pb2.TokenType.GroupReference,
                    groupref=root_pb2.GroupReferenceType.OpenCapture
                ),
                root_pb2.Token(
                    token="u",
                    type=root_pb2.TokenType.Character,
                    character="u"
                ),
                root_pb2.Token(
                    token="|",
                    type=root_pb2.TokenType.QuantifierModifier,
                    quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                ),
                root_pb2.Token(
                    token="l",
                    type=root_pb2.TokenType.Character,
                    character="l"
                ),
                root_pb2.Token(
                    token="l",
                    type=root_pb2.TokenType.Character,
                    character="l"
                ),
                root_pb2.Token(
                    token="?",
                    type=root_pb2.TokenType.QuantifierModifier,
                    quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                ),
                root_pb2.Token(
                    token="|",
                    type=root_pb2.TokenType.QuantifierModifier,
                    quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                ),
                root_pb2.Token(
                    token="l",
                    type=root_pb2.TokenType.Character,
                    character="l"
                ),
                root_pb2.Token(
                    token="|",
                    type=root_pb2.TokenType.QuantifierModifier,
                    quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                ),
                root_pb2.Token(
                    token="f",
                    type=root_pb2.TokenType.Character,
                    character="f"
                ),
                root_pb2.Token(
                    token=")",
                    type=root_pb2.TokenType.GroupReference,
                    groupref=root_pb2.GroupReferenceType.ClosedCapture
                ),
                root_pb2.Token(
                    token="?",
                    type=root_pb2.TokenType.QuantifierModifier,
                    quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                ),
                root_pb2.Token(
                    token="/",
                    type=root_pb2.TokenType.Anchor,
                    anchor=root_pb2.AnchorType.ForwardSlash
                ),
                root_pb2.Token(
                    token="i",
                    type=root_pb2.TokenType.Flag,
                    flag=root_pb2.FlagType.Ignore
                ),
            ],
            expressions=[
                root_pb2.Expression(
                    raw='/',
                    tokens=[
                        root_pb2.Token(
                            token=r"/",
                            type=root_pb2.TokenType.Anchor,
                            anchor=root_pb2.AnchorType.ForwardSlash
                        ),
                    ]
                ),
                root_pb2.Expression(
                    raw=r'/^(?:0x[a-f\d]+|0b[01]+|(?:\d+\.?\d*|\.\d+)(?:e[-+]?\d+)?)(u|ll?|l|f)?/i',
                    tokens=[
                        root_pb2.Token(
                            token=r"/",
                            type=root_pb2.TokenType.Anchor,
                            anchor=root_pb2.AnchorType.ForwardSlash
                        ),
                        root_pb2.Token(
                            token="^",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.SetNegation
                        ),
                        root_pb2.Token(
                            token=r"(",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.OpenCapture
                        ),
                        root_pb2.Token(
                            token=r"?:",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.NonCapturing
                        ),
                        root_pb2.Token(
                            token="0",
                            type=root_pb2.TokenType.Character,
                            character="0"
                        ),
                        root_pb2.Token(
                            token="x",
                            type=root_pb2.TokenType.Character,
                            character="x"
                        ),
                        root_pb2.Token(
                            token="[",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.OpenSet
                        ),
                        root_pb2.Token(
                            token="a-f",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.RangeSet
                        ),
                        root_pb2.Token(
                            token=r"\d",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.Digit
                        ),
                        root_pb2.Token(
                            token="]",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.CloseSet
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                        ),
                        root_pb2.Token(
                            token="|",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                        ),
                        root_pb2.Token(
                            token="0",
                            type=root_pb2.TokenType.Character,
                            character="0"
                        ),
                        root_pb2.Token(
                            token="b",
                            type=root_pb2.TokenType.Character,
                            character="b"
                        ),
                        root_pb2.Token(
                            token="[",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.OpenSet
                        ),
                        root_pb2.Token(
                            token="0",
                            type=root_pb2.TokenType.Character,
                            character="0"
                        ),
                        root_pb2.Token(
                            token="1",
                            type=root_pb2.TokenType.Character,
                            character="1"
                        ),
                        root_pb2.Token(
                            token="]",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.CloseSet
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                        ),
                        root_pb2.Token(
                            token="|",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                        ),
                        root_pb2.Token(
                            token=r"(",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.OpenCapture
                        ),
                        root_pb2.Token(
                            token="?:",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.NonCapturing
                        ),
                        root_pb2.Token(
                            token=r"\d",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.Digit
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                        ),
                        root_pb2.Token(
                            token="\.",
                            type=root_pb2.TokenType.Escape,
                            escape=root_pb2.EscapeType.Reserved
                        ),
                        root_pb2.Token(
                            token="?",
                            type=root_pb2.TokenType.QuantifierModifider,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                        ),
                        root_pb2.Token(
                            token=r"\d",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.Digit
                        ),
                        root_pb2.Token(
                            token="*",
                            type=root_pb2.TokenType.QuantifierModifider,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Star
                        ),
                        root_pb2.Token(
                            token=r"\.",
                            type=root_pb2.TokenType.Anchor,
                            anchor=root_pb2.AnchorType.ForwardSlash
                        ),
                        root_pb2.Token(
                            token=r"\d",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.Digit
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                        ),
                        root_pb2.Token(
                            token=")",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.CloseCapture
                        ),
                        root_pb2.Token(
                            token=r"(",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.OpenCapture
                        ),
                        root_pb2.Token(
                            token="?:",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.NonCapturing
                        ),
                        root_pb2.Token(
                            token="e",
                            type=root_pb2.TokenType.Character,
                            character="e"
                        ),
                        root_pb2.Token(
                            token="[",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.OpenSet
                        ),
                        root_pb2.Token(
                            token="-",
                            type=root_pb2.TokenType.Character,
                            character="-"
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.Character,
                            character="+"
                        ),
                        root_pb2.Token(
                            token="]",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.CloseSet
                        ),
                        root_pb2.Token(
                            token="?",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                        ),
                        root_pb2.Token(
                            token=r"\d",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.Digit
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                        ),
                        root_pb2.Token(
                            token=")",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.ClosedCapture
                        ),
                        root_pb2.Token(
                            token="?",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                        ),
                        root_pb2.Token(
                            token=")",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.ClosedCapture
                        ),
                        root_pb2.Token(
                            token=r"(",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.OpenCapture
                        ),
                        root_pb2.Token(
                            token="u",
                            type=root_pb2.TokenType.Character,
                            character="u"
                        ),
                        root_pb2.Token(
                            token="|",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                        ),
                        root_pb2.Token(
                            token="l",
                            type=root_pb2.TokenType.Character,
                            character="l"
                        ),
                        root_pb2.Token(
                            token="l",
                            type=root_pb2.TokenType.Character,
                            character="l"
                        ),
                        root_pb2.Token(
                            token="?",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                        ),
                        root_pb2.Token(
                            token="|",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                        ),
                        root_pb2.Token(
                            token="l",
                            type=root_pb2.TokenType.Character,
                            character="l"
                        ),
                        root_pb2.Token(
                            token="|",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                        ),
                        root_pb2.Token(
                            token="f",
                            type=root_pb2.TokenType.Character,
                            character="f"
                        ),
                        root_pb2.Token(
                            token=")",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.ClosedCapture
                        ),
                        root_pb2.Token(
                            token="?",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                        ),
                        root_pb2.Token(
                            token="/",
                            type=root_pb2.TokenType.Anchor,
                            anchor=root_pb2.AnchorType.ForwardSlash
                        ),
                        root_pb2.Token(
                            token="i",
                            type=root_pb2.TokenType.Flag,
                            flag=root_pb2.FlagType.Ignore
                        ),
                    ]
                ),
                root_pb2.Expression(
                    raw='[a-f\d]+',
                    tokens=[
                        root_pb2.Token(
                            token="[",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.OpenSet
                        ),
                        root_pb2.Token(
                            token="a-f",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.RangeSet
                        ),
                        root_pb2.Token(
                            token=r"\d",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.Digit
                        ),
                        root_pb2.Token(
                            token="]",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.CloseSet
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                        ),
                    ]
                ),
                root_pb2.Expression(
                    raw='0x',
                    tokens=[
                        root_pb2.Token(
                            token="0",
                            type=root_pb2.TokenType.Character,
                            character="0"
                        ),
                        root_pb2.Token(
                            token="x",
                            type=root_pb2.TokenType.Character,
                            character="x"
                        ),
                    ]
                ),
                root_pb2.Expression(
                    raw='0b',
                    tokens=[
                        root_pb2.Token(
                            token="0",
                            type=root_pb2.TokenType.Character,
                            character="0"
                        ),
                        root_pb2.Token(
                            token="b",
                            type=root_pb2.TokenType.Character,
                            character="b"
                        ),
                    ]
                ),
                root_pb2.Expression(
                    raw='"',
                    tokens=[
                        root_pb2.Token(
                            token='"',
                            type=root_pb2.TokenType.Character,
                            character='"'
                        ),
                    ]
                ),
                root_pb2.Expression(
                    raw='[a-f\d]+|0b',
                    tokens=[
                        root_pb2.Token(
                            token="[",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.OpenSet
                        ),
                        root_pb2.Token(
                            token="a-f",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.RangeSet
                        ),
                        root_pb2.Token(
                            token=r"\d",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.Digit
                        ),
                        root_pb2.Token(
                            token="]",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.CloseSet
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                        ),
                        root_pb2.Token(
                            token="|",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                        ),
                        root_pb2.Token(
                            token="0",
                            type=root_pb2.TokenType.Character,
                            character="0"
                        ),
                        root_pb2.Token(
                            token="b",
                            type=root_pb2.TokenType.Character,
                            character="b"
                        ),
                    ]
                ),
                root_pb2.Expression(
                    raw='[01]+|(?:\d+\.?\d*|\.\d+)',
                    tokens=[
                        root_pb2.Token(
                            token="[",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.OpenSet
                        ),
                        root_pb2.Token(
                            token="0",
                            type=root_pb2.TokenType.Character,
                            character="0"
                        ),
                        root_pb2.Token(
                            token="1",
                            type=root_pb2.TokenType.Character,
                            character="1"
                        ),
                        root_pb2.Token(
                            token="]",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.CloseSet
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                        ),
                        root_pb2.Token(
                            token="|",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                        ),
                        root_pb2.Token(
                            token=r"(",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.OpenCapture
                        ),
                        root_pb2.Token(
                            token="?:",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.NonCapturing
                        ),
                        root_pb2.Token(
                            token=r"\d",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.Digit
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                        ),
                        root_pb2.Token(
                            token="|",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                        ),
                        root_pb2.Token(
                            token="\.",
                            type=root_pb2.TokenType.Escape,
                            escape=root_pb2.EscapeType.Reserved
                        ),
                        root_pb2.Token(
                            token="?",
                            type=root_pb2.TokenType.QuantifierModifider,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                        ),
                        root_pb2.Token(
                            token=r"\d",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.Digit
                        ),
                        root_pb2.Token(
                            token="*",
                            type=root_pb2.TokenType.QuantifierModifider,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Star
                        ),
                        root_pb2.Token(
                            token=r"\.",
                            type=root_pb2.TokenType.Anchor,
                            anchor=root_pb2.AnchorType.ForwardSlash
                        ),
                        root_pb2.Token(
                            token=r"\d",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.Digit
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                        ),
                        root_pb2.Token(
                            token=")",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.CloseCapture
                        ),
                        root_pb2.Token(
                            token=r"(",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.OpenCapture
                        ),
                        root_pb2.Token(
                            token="?:",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.NonCapturing
                        ),
                        root_pb2.Token(
                            token="e",
                            type=root_pb2.TokenType.Character,
                            character="e"
                        ),
                        root_pb2.Token(
                            token="[",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.OpenSet
                        ),
                        root_pb2.Token(
                            token="-",
                            type=root_pb2.TokenType.Character,
                            character="-"
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.Character,
                            character="+"
                        ),
                        root_pb2.Token(
                            token="]",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.CloseSet
                        ),
                        root_pb2.Token(
                            token="?",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                        ),
                        root_pb2.Token(
                            token=r"\d",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.Digit
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                        ),
                        root_pb2.Token(
                            token=r")",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.CloseCapture
                        ),
                    ]
                ),
                root_pb2.Expression(
                    raw='(?:e[-+]?\d+)?)(u | ll? | l | f)?',
                    tokens=[
                        root_pb2.Token(
                            token=r"(",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.OpenCapture
                        ),
                        root_pb2.Token(
                            token="?:",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.NonCapturing
                        ),
                        root_pb2.Token(
                            token="e",
                            type=root_pb2.TokenType.Character,
                            character="e"
                        ),
                        root_pb2.Token(
                            token="[",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.OpenSet
                        ),
                        root_pb2.Token(
                            token="-",
                            type=root_pb2.TokenType.Character,
                            character="-"
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.Character,
                            character="+"
                        ),
                        root_pb2.Token(
                            token="]",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.CloseSet
                        ),
                        root_pb2.Token(
                            token="?",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                        ),
                        root_pb2.Token(
                            token=r"\d",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.Digit
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                        ),
                        root_pb2.Token(
                            token=")",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.ClosedCapture
                        ),
                        root_pb2.Token(
                            token="?",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                        ),
                        root_pb2.Token(
                            token=")",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.ClosedCapture
                        ),
                        root_pb2.Token(
                            token=r"(",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.OpenCapture
                        ),
                        root_pb2.Token(
                            token="u",
                            type=root_pb2.TokenType.Character,
                            character="u"
                        ),
                        root_pb2.Token(
                            token="|",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                        ),
                        root_pb2.Token(
                            token="l",
                            type=root_pb2.TokenType.Character,
                            character="l"
                        ),
                        root_pb2.Token(
                            token="l",
                            type=root_pb2.TokenType.Character,
                            character="l"
                        ),
                        root_pb2.Token(
                            token="?",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                        ),
                        root_pb2.Token(
                            token="|",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                        ),
                        root_pb2.Token(
                            token="l",
                            type=root_pb2.TokenType.Character,
                            character="l"
                        ),
                        root_pb2.Token(
                            token="|",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                        ),
                        root_pb2.Token(
                            token="f",
                            type=root_pb2.TokenType.Character,
                            character="f"
                        ),
                        root_pb2.Token(
                            token=")",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.ClosedCapture
                        ),
                        root_pb2.Token(
                            token="?",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                        ),
                    ]
                ),
                root_pb2.Expression(
                    raw='(?:e[-+]?\d+)?)',
                    tokens=[
                        root_pb2.Token(
                            token=r"(",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.OpenCapture
                        ),
                        root_pb2.Token(
                            token="?:",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.NonCapturing
                        ),
                        root_pb2.Token(
                            token="e",
                            type=root_pb2.TokenType.Character,
                            character="e"
                        ),
                        root_pb2.Token(
                            token="[",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.OpenSet
                        ),
                        root_pb2.Token(
                            token="-",
                            type=root_pb2.TokenType.Character,
                            character="-"
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.Character,
                            character="+"
                        ),
                        root_pb2.Token(
                            token="]",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.CloseSet
                        ),
                        root_pb2.Token(
                            token="?",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                        ),
                        root_pb2.Token(
                            token=r"\d",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.Digit
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                        ),
                        root_pb2.Token(
                            token=")",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.ClosedCapture
                        ),
                        root_pb2.Token(
                            token="?",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                        ),
                        root_pb2.Token(
                            token=")",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.ClosedCapture
                        ),
                    ]
                ),
                root_pb2.Expression(
                    raw='(?:\d+\.?\d * | \.\d+)',
                    tokens=[
                        root_pb2.Token(
                            token=r"(",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.OpenCapture
                        ),
                        root_pb2.Token(
                            token="?:",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.NonCapturing
                        ),
                        root_pb2.Token(
                            token=r"\d",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.Digit
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                        ),
                        root_pb2.Token(
                            token="\.",
                            type=root_pb2.TokenType.Escape,
                            escape=root_pb2.EscapeType.Reserved
                        ),
                        root_pb2.Token(
                            token="?",
                            type=root_pb2.TokenType.QuantifierModifider,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                        ),
                        root_pb2.Token(
                            token=r"\d",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.Digit
                        ),
                        root_pb2.Token(
                            token="*",
                            type=root_pb2.TokenType.QuantifierModifider,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Star
                        ),
                        root_pb2.Token(
                            token="|",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                        ),
                        root_pb2.Token(
                            token=r"\.",
                            type=root_pb2.TokenType.Anchor,
                            anchor=root_pb2.AnchorType.ForwardSlash
                        ),
                        root_pb2.Token(
                            token=r"\d",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.Digit
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                        ),
                        root_pb2.Token(
                            token=")",
                            type=root_pb2.TokenType.GroupReference,
                            groupref=root_pb2.GroupReferenceType.CloseCapture
                        ),
                    ]
                ),
                root_pb2.Expression(
                    raw='[-+]?',
                    tokens=[
                        root_pb2.Token(
                            token="[",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.OpenSet
                        ),
                        root_pb2.Token(
                            token="-",
                            type=root_pb2.TokenType.Character,
                            character="-"
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.Character,
                            character="+"
                        ),
                        root_pb2.Token(
                            token="]",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.CloseSet
                        ),
                        root_pb2.Token(
                            token="?",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                        ),
                    ]
                ),
                root_pb2.Expression(
                    raw='\d+',
                    tokens=[
                        root_pb2.Token(
                            token=r"\d",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.Digit
                        ),
                        root_pb2.Token(
                            token="+",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Plus
                        ),
                    ]
                ),
                root_pb2.Expression(
                    raw='\.?',
                    tokens=[
                        root_pb2.Token(
                            token="\.",
                            type=root_pb2.TokenType.Escape,
                            escape=root_pb2.EscapeType.Reserved
                        ),
                        root_pb2.Token(
                            token="?",
                            type=root_pb2.TokenType.QuantifierModifider,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                        ),
                    ]
                ),
                root_pb2.Expression(
                    raw='\d*|\.',
                    tokens=[
                        root_pb2.Token(
                            token=r"\d",
                            type=root_pb2.TokenType.CharacterClass,
                            characterclass=root_pb2.CharacterClassType.Digit
                        ),
                        root_pb2.Token(
                            token="*",
                            type=root_pb2.TokenType.QuantifierModifider,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Star
                        ),
                        root_pb2.Token(
                            token="|",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                        ),
                        root_pb2.Token(
                            token=r"\.",
                            type=root_pb2.TokenType.Anchor,
                            anchor=root_pb2.AnchorType.ForwardSlash
                        ),
                    ]
                ),
                root_pb2.Expression(
                    raw='l?|l',
                    tokens=[
                        root_pb2.Token(
                            token="l",
                            type=root_pb2.TokenType.Character,
                            character="l"
                        ),
                        root_pb2.Token(
                            token="?",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.Optional
                        ),
                        root_pb2.Token(
                            token="|",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                        ),
                        root_pb2.Token(
                            token="l",
                            type=root_pb2.TokenType.Character,
                            character="l"
                        ),
                    ]
                ),
                root_pb2.Expression(
                    raw='u|l',
                    tokens=[
                        root_pb2.Token(
                            token="u",
                            type=root_pb2.TokenType.Character,
                            character="u"
                        ),
                        root_pb2.Token(
                            token="|",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                        ),
                        root_pb2.Token(
                            token="l",
                            type=root_pb2.TokenType.Character,
                            character="l"
                        ),
                    ]
                ),
                root_pb2.Expression(
                    raw='l|f',
                    tokens=[
                        root_pb2.Token(
                            token="l",
                            type=root_pb2.TokenType.Character,
                            character="l"
                        ),
                        root_pb2.Token(
                            token="|",
                            type=root_pb2.TokenType.QuantifierModifier,
                            quantifiermodifier=root_pb2.QuantifierModifierType.AlternationPipe
                        ),
                        root_pb2.Token(
                            token="f",
                            type=root_pb2.TokenType.Character,
                            character="f"
                        ),
                    ]
                ),
                root_pb2.Expression(
                    raw='/',
                    tokens=[
                        root_pb2.Token(
                            token="/",
                            type=root_pb2.TokenType.Anchor,
                            anchor=root_pb2.AnchorType.ForwardSlash
                        ),
                    ]
                )
            ]
        ))
    }